unit uDMConnection;

interface

uses
  Classes, DB, DBTables;

type
  TDMConnection = class(TDataModule)
    DBConnection: TDatabase;
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }

    FNextCust: TQuery;
    FNextOrder: TQuery;
    FNextItem: TQuery;

    function NextVal(DataSet: TDataSet; const FieldName: string): integer;

  public
    { Public declarations }

    function NextCust: integer;
    function NextOrder: integer;
    function NextItem(const OrderNo: integer): integer;

  end;

// Simple Singleton - Declare a globally available function to return the data
//                    module
function gDMConnection: TDMConnection;

{var
  DMConnection: TDMConnection;}

implementation

uses
  Forms
  ;

var
  DM: TDMConnection;

function gDMConnection: TDMConnection;
begin
  if not Assigned(DM) then
    DM := TDMConnection.Create(Application);
  Result := DM;
end;

{$R *.dfm}

procedure TDMConnection.DataModuleCreate(Sender: TObject);
begin
  // Create the "next record" queries
  FNextCust := TQuery.Create(Self);
  FNextCust.DatabaseName := DBConnection.DatabaseName;
  FNextCust.SQL.Clear;
  FNextCust.SQL.Add('SELECT * FROM nextcust');
  FNextCust.RequestLive := True;
  FNextOrder := TQuery.Create(Self);
  FNextOrder.DatabaseName := DBConnection.DatabaseName;
  FNextOrder.SQL.Clear;
  FNextOrder.SQL.Add('SELECT * FROM nextord');
  FNextOrder.RequestLive := True;
  FNextItem := TQuery.Create(Self);
  FNextItem.DatabaseName := DBConnection.DatabaseName;
  FNextItem.SQL.Clear;
  FNextItem.SQL.Add('SELECT (MAX(ItemNo) + 1) AS nextitem FROM Items WHERE OrderNo = :OrderNo');
  // Make sure the database is available
  if not DBConnection.Connected then
    DBConnection.Connected := True;
end;

function TDMConnection.NextCust: integer;
begin
  Result := NextVal(FNextCust, 'NewCust');
end;

function TDMConnection.NextItem(const OrderNo: integer): integer;
begin
  if FNextItem.Active then
    FNextItem.Close;
  FNextItem.ParamByName('OrderNo').AsInteger := OrderNo;
  FNextItem.Open;
  if (not FNextItem.IsEmpty) and
     (not FNextItem.FieldByName('nextitem').IsNull) then
  begin
    Result := FNextItem.FieldByName('nextitem').AsInteger;
  end
  else
  begin
    Result := 1;
  end;
end;

function TDMConnection.NextOrder: integer;
begin
  Result := NextVal(FNextOrder, 'NewKey');
end;

// Get a value from the database, increment it & write it back. Return the
// original value (as read from the database)
function TDMConnection.NextVal(DataSet: TDataSet; const FieldName: string): integer;
var
  nextval: integer;
begin
  nextval := -1;
  // Get the current value of nextval and increment it
  // Start a transaction
  DBConnection.StartTransaction;
  try
    // Get the current value from the database. There should only be one record
    DataSet.Open;
    try
      if not DataSet.IsEmpty then
      begin
        nextval := DataSet.FieldByName(FieldName).AsInteger;
        DataSet.Edit;
      end
      else
      begin
        // No record in table yet - initialise it
        nextval := 1;
        DataSet.Insert;
      end;
      DataSet.FieldByName(FieldName).AsInteger := nextval + 1;
      DataSet.Post;
    finally
      DataSet.Close;
    end;
    DBConnection.Commit;
  except
    DBConnection.Rollback;
  end;
  Result := nextval;
end;

initialization
begin
  DM := TDMConnection.Create(Application);
end;

end.
